# -*- coding:utf-8 -*-

"""
general exception used in this module
"""


class CoinCheckApiException(Exception):

    def __init__(self, message):
        super(CoinCheckApiException, self).__init__(message)



# -*- coding:utf-8 -*-
import coincheck_api.auth
from coincheck_api.clients.api_client import ApiClient
from logging import getLogger
logger = getLogger(__name__)

try:
    # python2
    from urllib import urlencode
except ImportError:
    # python3
    from urllib.parse import urlencode


class PrivateApiClient(ApiClient):
    """
    In order to use private APIs (e.g. Order, Account balance check)
    You should create an API key in advance.
    https://coincheck.com/ja/documents/exchange/api#auth
    """

    def __init__(self, access_key, secret):
        self.access_key = access_key
        self.secret = secret

    def execute_private_api(self, path, method, data=None):
        """
        execute an api which needs authentication headers
        :param path: API path (e.g. /api/ticker )
        :param method: HTTP method in string. "GET", "POST", "PUT", "DELETE"
        :param data: parameter dictionary to send. e.g. ({"key1":"value1", "key2":"value2"}
        :return: response in json-deserialized dictionary object
        """
        url = ApiClient.base_url + path
        body = ""
        if method in ["POST", "PUT", "DELETE"] and data is not None:
            body = urlencode(data)
        headers = coincheck_api.auth.create_auth_headers(
            url,
            access_key=self.access_key,
            secret=self.secret,
            body=body)

        return self.execute_http_call(path, method, headers, data)



# -*- coding:utf-8 -*-
from coincheck_api.clients.private_api_client import PrivateApiClient
from coincheck_api.exception import CoinCheckApiException


class BorrowApiClient(PrivateApiClient):

    def __init__(self, access_key, secret):
        PrivateApiClient.__init__(self, access_key, secret)

    def borrow_currency(self, amount, currency):
        """
        borrow bitcoin/etherium from coincheck
        :param amount: amount to borrow
        :param currency: currency to borrow
        :return: borrowed information (rate etc.)
        """
        if currency not in ["btc", "eth"]:
            raise CoinCheckApiException("currency to borrow should be \"btc\" or \"etc\"")

        data = {"amount": amount, "currency": currency}
        return self.execute_private_api("/api/lending/borrows", "POST", data=data)

    def get_borrowings(self):
        """
        get borrowings information
        :return: borrowings information
        """
        return self.execute_private_api("/api/lending/borrows/matches", "GET")

    def repay_currency(self, borrow_id):
        """
        repay money to coincheck
        :param borrow_id: id issued when borrowed
        :return: succeeded or not
        """
        return self.execute_private_api("api/lending/borrows/{}/repay".format(borrow_id), "POST")



# -*- coding:utf-8 -*-

import json
from logging import getLogger

from coincheck_api.clients.private_api_client import PrivateApiClient
from coincheck_api.exception import CoinCheckApiException

logger = getLogger(__name__)


class OrderApiClient(PrivateApiClient):

    def __init__(self, access_key, secret):
        PrivateApiClient.__init__(self, access_key, secret)

    def buy(self, rate, btc_amount, dry_run=False):
        """
        buy bitcoin at limit
        :param rate: rate of btc/jpy (e.g. 1200000)
        :param btc_amount: amount of bitcoin (e.g. 0.1)
        :param dry_run: for debug. when true, order is not actually executed
        :return: order information
        """
        if rate is None:
            raise CoinCheckApiException("you need to specify rate when order at a limit price")
        return self.order(order_type="buy", rate=rate, amount=btc_amount, dry_run=dry_run)

    def sell(self, rate, btc_amount, dry_run=False):
        """
        sell bitcoin at limit
        :param rate: rate of btc/jpy (e.g. 1200000)
        :param btc_amount: amount of bitcoin (e.g. 0.1)
        :param dry_run: for debug. when true, order is not actually executed
        :return: order information
        """
        if rate is None:
            raise CoinCheckApiException("you need to specify rate when order at a limit price")
        return self.order(order_type="sell", rate=rate, amount=btc_amount, dry_run=dry_run)

    def market_buy(self, jpy_amount, dry_run=False):
        """
        buy bitcoin without limit.
        [CAUTION] !!!!!! you need to specify "JPY" amount, not "BTC" amount !!!!!!!!
        it's due to the weird api spec of coincheck
        https://coincheck.com/ja/documents/exchange/api#order-new
        :param jpy_amount: amount of JPY to buy without llimit
        :param dry_run: for debug. when true, order is not actually executed
        :return: order information
        """
        return self.order("market_buy", rate=None, amount=None, market_buy_amount=jpy_amount, dry_run=dry_run)

    def market_sell(self, btc_amount, dry_run=False):
        """
        sell bitcoin without limit.
        :param btc_amount: amount of btc to sell without limit
        :param dry_run: for debug. when true, order is not actually executed
        :return: order information
        """
        return self.order("market_sell", rate=None, amount=btc_amount, dry_run=dry_run)

    def leverage_buy(self, rate, btc_amount, dry_run=False):
        """
        buy bitcoin at a certain price in a leveraged order
        :param rate: rate of btc/jpy (e.g. 1200000)
        :param btc_amount: amount of bitcoin to buy
        :param dry_run: for debug. when true, order is not actually executed
        :return: order information
        """
        return self.order("leverage_buy", rate=rate, amount=btc_amount, dry_run=dry_run)

    def leverage_sell(self, rate, btc_amount, dry_run=False):
        """
        sell bitcoin at a certain price in a leveraged order
        :param rate: rate of btc/jpy (e.g. 1200000)
        :param btc_amount: amount of bitcoin to sell
        :param dry_run: for debug. when true, order is not actually executed
        :return: order information
        """
        return self.order("leverage_sell", rate=rate, amount=btc_amount, dry_run=dry_run)

    def leverage_market_buy(self, btc_amount, dry_run=False):
        """
        buy bitcoin without limit in a leverage order
        :param btc_amount: amount of bitcoin to buy
        :param dry_run: for debug. when true, order is not actually executed
        :return: order information
        """
        return self.order("leverage_buy", rate=None, amount=btc_amount, dry_run=dry_run)

    def leverage_market_sell(self, btc_amount, dry_run=False):
        """
        sell bitcoin without limit in a leverage order
        :param btc_amount: amount of bitcoin to sell
        :param dry_run: for debug. when true, order is not actually executed
        :return: order information
        """
        return self.order("leverage_sell", rate=None, amount=btc_amount, dry_run=dry_run)

    def close_long(self, rate, btc_amount, position_id, dry_run=False):
        """
        close a position by selling bitcoin at a certain price
        :param rate: rate of btc/jpy (e.g. 1200000)
        :param btc_amount: amount of bitcoin to sell
        :param position_id: id issued when ordered
        :param dry_run: for debug. when true, order is not actually executed
        :return:
        """
        return self.order("close_long", rate=rate, amount=btc_amount, position_id=position_id, dry_run=dry_run)

    def close_short(self, rate, btc_amount, position_id, dry_run=False):
        """
        close a position by selling bitcoin at a certain price
        :param rate: rate of btc/jpy (e.g. 1200000)
        :param btc_amount: amount of bitcoin to sell
        :param position_id: id issued when ordered
        :param dry_run: for debug. when true, order is not actually executed
        :return:
        """
        return self.order("close_short", rate=rate, amount=btc_amount, position_id=position_id, dry_run=dry_run)

    def market_close_long(self, btc_amount, position_id, dry_run=False):
        """
        close a position by selling bitcoin without limit
        :param btc_amount: amount of bitcoin to sell
        :param position_id: id issued when ordered
        :param dry_run: for debug. when true, order is not actually executed
        :return:
        """
        return self.order("close_long", rate=None, amount=btc_amount, position_id=position_id, dry_run=dry_run)

    def market_close_short(self, btc_amount, position_id, dry_run=False):
        """
        close a position by selling bitcoin without limit
        :param btc_amount: amount of bitcoin to sell
        :param position_id: id issued when ordered
        :param dry_run: for debug. when true, order is not actually executed
        :return:
        """
        return self.order("close_short", rate=None, amount=btc_amount, position_id=position_id, dry_run=dry_run)

    def order(self, order_type, rate, amount, market_buy_amount=None,
              position_id=None, stop_loss_rate=None, pair="btc_jpy", dry_run=False):
        """
        order
        :param order_type: one of ["buy", "sell", "market_buy", "market_sell",
                              "leverage_buy", "leverage_sell", "close_long", "close_short"]
        :param rate: rate to order
        :param amount: amount of bitcoin to order
        :param market_buy_amount: amount of JPY to buy without limit
        :param position_id: position_id to close
        :param stop_loss_rate: stop_loss_rate to order.
        for instance, if order_type="buy", rate=48400 and stop_loss_rate=48300,
        when btc/jpy exceeds 48300, order is executed.
        if order_type="sell", rate=None and stop_loss_rate=48400,
        when btc/jpy falls below 48400, sell order is executed without limit.
        cf. https://coincheck.com/ja/info/faq_posts/35
        :param pair: currency pair. As of 2018-05, only "btc_jpy" is supported.
        :param dry_run: for debug. when it's true, order is not executed actually.
        :return:
        """
        if order_type not in ["buy", "sell", "market_buy", "market_sell",
                              "leverage_buy", "leverage_sell", "close_long", "close_short"]:
            raise CoinCheckApiException("""order_type should be one of [buy, sell, 
            market_buy, market_sell,leverage_buy, leverage_sell, close_long, close_short""")

        data = {"order_type": order_type, "pair": pair}
        if rate is not None:
            data["rate"] = rate
        if amount is not None:
            data["amount"] = amount
        if market_buy_amount is not None:
            data["market_buy_amount"] = market_buy_amount
        if position_id is not None:
            data["position_id"] = position_id
        if stop_loss_rate is not None:
            data["stop_loss_rate"] = stop_loss_rate

        if dry_run:
            logger.info("[dry run] order is executed. request data={}".format(json.dumps(data)))
            return {"success": True, "id": 12345, "rate": str(rate), "amount": str(amount),
                    "order_type": order_type, "stop_loss_rate": stop_loss_rate, "pair": pair,
                    "created_at": "2015-01-10T05:55:38.000Z"}
        else:
            response = self.execute_private_api("/api/exchange/orders", "POST", data=data)
            logger.info("order is executed. request data={}".format(data))
            return response

    def get_open_orders(self):
        """
        get open orders
        :return: open orders
        """
        return self.execute_private_api("/api/exchange/orders/opens", "GET")

    def cancel_open_order(self, order_id):
        """
        cancel an open order
        :param order_id: id of the order the user wants to cancel
        :return: canceled order id
        """
        return self.execute_private_api("/api/exchange/orders/{}".format(order_id), "DELETE")

    def get_order_transactions(self):
        """
        get recent orders' information
        :return: recent orders' information
        """
        return self.execute_private_api("/api/exchange/orders/transactions", "GET")

    def get_order_transactions_pagination(self, limit=None, order=None, start_id=None, end_id=None):
        """
        get recent orders' information with pagination functionality
        :param limit: the number of items to get
        :param order: "desc" or "asc"
        :param start_id: get items after the id
        :param end_id: get items before the id
        :return: recent orders' information
        """
        data = {"limit": limit, "order": order, "starting_after": start_id, "ending_before": end_id}
        return self.execute_private_api("/api/exchange/orders/transactions_pagination", "GET", data=data)

    def get_leverage_positions(self, status, limit=None, order=None, start_id=None, end_id=None):
        """
        get positions in a certain status with pagination functionality
        :param status: "open" or "closed"
        :param limit: the number of items to get
        :param order: "desc" or "asc"
        :param start_id: get items after the id
        :param end_id: get items before the id
        :return: positions in a certain status
        """
        if status not in ["open", "closed"]:
            raise CoinCheckApiException("status should be \"open\" or \"closed\".")
        data = {"limit": limit, "order": order, "starting_after": start_id, "ending_before": end_id}
        return self.execute_private_api("/api/exchange/leverage/positions", "GET", data=data)



# -*- coding:utf-8 -*-
from coincheck_api.clients.private_api_client import PrivateApiClient
from coincheck_api.exception import CoinCheckApiException


class WithdrawApiClient(PrivateApiClient):

    def __init__(self, access_key, secret):
        PrivateApiClient.__init__(self, access_key, secret)

    def get_bank_account(self):
        """
        get bank account registered to coincheck
        :return: bank account information
        """
        return self.execute_private_api("/api/bank_accounts", "GET")

    def register_bank_account(self, bank_name, branch_name, bank_account_type, number, name):
        """
        register a bank account used to send money from coincheck
        :param bank_name: bank name
        :param branch_name: branch name
        :param bank_account_type: "futsu"(savings account) or "toza"(checking account)
        :param number: account number
        :param name: name of the bank account holder
        :return: registered account information
        """
        if bank_account_type not in ["futsu", "toza"]:
            raise CoinCheckApiException(
                "bank_account_type should be \"futsu\"(savings account) or \"toza\"(checking account)")
        data = {"bank_name": bank_name, "branch_name": branch_name, "bank_account_type": bank_account_type,
                "number": number, "name": name}
        return self.execute_private_api("/api/bank_accounts", "POST", data=data)

    def delete_bank_account(self, bank_id):
        """
        delete a registration of a bank account
        :param bank_id: id of the bank
        :return: succeeded to delete or not
        """
        return self.execute_private_api("/api/bank_accounts/{}".format(bank_id), "DELETE")

    def get_withdraws(self, limit=None, order=None, start_id=None, end_id=None):
        """
        get history of requests to withdraw JPY with pagination functionality
        :param limit: the number of items to get
        :param order: "desc" or "asc"
        :param start_id: get items after the id
        :param end_id: get items before the id
        :return: history of requests to withdraw JPY
        """
        data = {"limit": limit, "order": order, "starting_after": start_id, "ending_before": end_id}
        return self.execute_private_api("/api/withdraws", "GET", data=data)

    def request_withdraw(self, bank_account_id, amount, currency="JPY"):
        """
        request to withdraw JPY
        :param bank_account_id: bank id
        :param amount: amount to withdraw
        :param currency: currency to withdraw. As of 2018-05, only "JPY" is supported.
        :return: result of the request
        """
        data = {"bank_account_id": bank_account_id, "amount": amount, "currency": currency}
        return self.execute_private_api("/api/withdraws", "POST", data=data)

    def cancel_withdraw(self, withdraw_request_id):
        """
        cancel a request to withdraw.
        the request only in "pending" status can be canceled.
        :param withdraw_request_id: id issued when requested to withdraw
        :return: cancel is succeeded or not
        """
        return self.execute_private_api("api/withdraws/{}".format(withdraw_request_id), "DELETE")



# -*- coding:utf-8 -*-
from coincheck_api.clients.api_client import ApiClient
from coincheck_api.exception import CoinCheckApiException


class PublicApiClient(ApiClient):

    def __init__(self):
        pass

    def get_btc_ticker(self):
        """
        get ticker information of Bitcoin
        :return: ticker information of Bitcoin
        """
        return self.execute_http_call("/api/ticker", "GET", headers=None)

    def get_trades(self, pair="btc_jpy"):
        """
        get latest information about trades of a certain currency pair.
        :param pair: currency pair joined by underscore. As of 2018/05, only "btc_jpy" is supported.
        :return:
        """
        return self.execute_http_call("/api/trades?pair={}".format(pair), "GET", headers=None)

    def get_order_books(self):
        """
        get order books of bitcoin
        :return: order books of bitcoin
        """
        return self.execute_http_call("/api/order_books", "GET", headers=None)

    def get_exchange_rate(self, order_type, amount=None, price=None, pair="btc_jpy"):
        """
        get exchange rate of a certain currency pair.
        :param order_type: "sell" or "buy"
        :param pair: currency pair joined by underscore. At the timing of 2018/05,
        only "btc_jpy" is supported.
        :param amount: amount of the currency to sell/buy. (e.g. 0.1)
        :param price: price to sell/buy in JPY. (e.g. 10000 [yen])
        :return: information about exchange rate
        """
        if order_type not in ["sell", "buy"]:
            raise CoinCheckApiException("order_type should be \"sell\" or \"buy\"")
        if amount is not None and price is not None:
            raise CoinCheckApiException("only one of \"amount\" and \"price\" can be provided ")

        query = "order_type={}&pair={}".format(order_type, pair)
        if amount is not None:
            query += "&amount={}".format(amount)
        else:
            query += "&price={}".format(price)

        return self.execute_http_call("/api/exchange/orders/rate?{}".format(query), "GET", headers=None)

    def get_sale_rate(self, pair):
        """
        get a rate of a certain pair(e.g. btc_jpy) sold by coincheck.
        if you enter "btc_jpy" as the pair and get 60000, it means you can buy 1 bitcoin for 60000 yen.

        :param pair: currency pair joined by underscore
        (e.g. "btc_jpy", "eth_jpy", "etc_jpy", "lsk_jpy", "fct_jpy", "xmr_jpy", "rep_jpy",
        "xrp_jpy", "zec_jpy", "xem_jpy", "ltc_jpy", "dash_jpy", "bch_jpy" )
        :return: rate value in floating value.
        """
        response = self.execute_http_call("/api/rate/{}".format(pair), "GET")
        return float(response["rate"])



# -*- coding:utf-8 -*-
import json
from logging import getLogger

import requests

from coincheck_api.exception import CoinCheckApiException

logger = getLogger(__name__)


class ApiClient:
    base_url = "https://coincheck.com"

    def __init__(self):
        pass

    def execute_http_call(self, path, method, headers=None, data=None):
        """
        :param path: API path (e.g. /api/ticker )
        :param method: HTTP method in string. "GET", "POST", "PUT", "DELETE"
        :param headers: HTTP headers dictionary
        :param data: parameter dictionary to send. e.g. ({"key1":"value1", "key2":"value2"}
        :return: response in json-deserialized dictionary object
        """
        url = self.base_url + path

        logger.debug("HTTP call executed. url={}, method={}, headers={}".format(url, method, headers))

        # execute an HTTP call
        response = requests.request(method, url, headers=headers, data=data)
        logger.debug("HTTP status code of response={} {}".format(response.status_code, response.content))

        response_body = json.loads(response.content)
        # success
        if 200 <= response.status_code < 400:
            return response_body
        # failure
        else:
            raise CoinCheckApiException(
                "API execute failure. response status_code={}, headers={}, body={}".format(
                    response.status_code, response.headers, response.content
                ))



# -*- coding:utf-8 -*-
from logging import getLogger

from coincheck_api.clients.private_api_client import PrivateApiClient

logger = getLogger(__name__)


class AccountApiClient(PrivateApiClient):

    def __init__(self, access_key, secret):
        PrivateApiClient.__init__(self, access_key, secret)

    def get_account_balance(self):
        """
        get balance of a account
        :return: account balance
        """
        return self.execute_private_api("/api/accounts/balance", "GET")

    def get_account_leverage_balance(self):
        """
        get balance of a leverage account
        :return: leverage_account balance
        """
        return self.execute_private_api("/api/accounts/leverage_balance", "GET")

    def send_money(self, address, amount, dry_run=False):
        """
        send bitcoin to a certain address
        :param address:
        :param amount:
        :param dry_run: for debug. when true, sending money is not actually executed
        :return:
        """
        payload = {'address': address, 'amount': amount}
        if dry_run:
            logger.info("[dry run] sending money is executed. address={}, amount={}".format(address, amount))
            return { "success": True, "id": "276", "address": address, "amount": amount, "fee": "0.002" }
        else:
            logger.info("sending money is executed. address={}, amount={}".format(address, amount))
            return self.execute_private_api("/api/send_money", "POST", data=payload)

    def get_send_money_history(self, currency="btc"):
        """
        get the history of a certain sent currency
        :param currency: currency that you want to see the history. As of 2018-05, only "btc" is supported
        :return: history
        """
        return self.execute_private_api("/api/send_money?currency={}".format(currency), "GET")

    def get_deposit_money_history(self, currency="btc"):
        """
        get history of certain deposited currency
        :param currency: currency that you want to see the history. As of 2018-05, only "btc" is supported
        :return: history
        """
        return self.execute_private_api("/api/deposit_money?currency={}".format(currency), "GET")

    def deposit_money_fast(self, receipt_id, dry_run=False):
        """
        deposi bitcoin quickly with some charge
        :param receipt_id: an id issued when received the money
        :param dry_run: for debug. when true, sending money is not actually executed
        :return: sending money is succeeded or not
        """
        if dry_run:
            logger.info("[dry run] deposit money (fast) is executed. receipt_id={}".format(receipt_id))
            return {"success": True}
        else:
            logger.info("deposit money (fast) is executed. receipt_id={}".format(receipt_id))
            return self.execute_private_api("/api/deposit_money/{}/fast".format(receipt_id), "POST")

    def get_account(self):
        """
        show coincheck account information
        :return: account information
        """
        return self.execute_private_api("/api/accounts", "GET")



# -*- coding:utf-8 -*-
from logging import getLogger

from coincheck_api.clients.account_api_client import AccountApiClient
from coincheck_api.clients.borrow_api_client import BorrowApiClient
from coincheck_api.clients.order_api_client import OrderApiClient
from coincheck_api.clients.public_api_client import PublicApiClient
from coincheck_api.clients.transfer_api_client import TransferApiClient
from coincheck_api.clients.withdraw_api_client import WithdrawApiClient

logger = getLogger(__name__)


class Client(PublicApiClient, OrderApiClient, AccountApiClient, BorrowApiClient, WithdrawApiClient, TransferApiClient):
    """
    In order to use private APIs (e.g. Order, Account balance check)
    You should create an API key in advance.
    https://coincheck.com/ja/documents/exchange/api#auth
    """

    def __init__(self, access_key, secret):
        PublicApiClient.__init__(self)
        OrderApiClient.__init__(self, access_key, secret)



# -*- coding:utf-8 -*-
import hashlib
import hmac
import time


def create_auth_headers(url, access_key, secret, body=None):
    """
    get HTTP headers for API authentication
    :param url: API url. (e.g. https://coincheck.com/api/accounts/balance )
    :param access_key: Access Key string for API authentication
    :param secret: Secret Access Key string for API authentication
    :return: HTTP header dictionary
    """
    current_millis = str(int(round(time.time() * 1000)))
    message = current_millis + url + body
    signature = hmac.new(secret.encode("utf-8"), message.encode("utf-8"), hashlib.sha256).hexdigest()
    headers = {
        "ACCESS-KEY": access_key,
        "ACCESS-NONCE": current_millis,
        "ACCESS-SIGNATURE": signature
    }
    return headers



# -*- coding: utf-8 -*-

from io import open
from setuptools import setup, find_packages

with open('README.md', encoding="utf-8") as f:
    readme = f.read()

with open('LICENSE', encoding="utf-8") as f:
    license = f.read()

setup(
    name='coincheck_api',
    version='0.8.1',
    description='A simple library to use CoinCheck API',
    long_description=readme,
    author='dakimura',
    author_email='dakimura@example.com',
    url='https://github.com/dakimura/coincheck_api',
    license=license,
    packages=find_packages(exclude=('tests', 'docs'))
)


# -*- coding:utf-8 -*-
from coincheck_api.clients.private_api_client import PrivateApiClient
from coincheck_api.exception import CoinCheckApiException


class TransferApiClient(PrivateApiClient):

    def __init__(self, access_key, secret):
        PrivateApiClient.__init__(self, access_key, secret)

    def transfer_to_leverage_account(self, amount, currency="JPY"):
        """
        transfer money to leverage account
        :param amount: amount to transfer
        :param currency: currency to transfer. As of 2018-05, only "JPY" is supported
        :return: succeeded or not
        """
        if currency not in ["JPY"]:
            raise CoinCheckApiException("currency should be \"JPY\", currently.")
        data = {"amount": amount, "currency": currency}
        return self.execute_private_api("/api/exchange/transfers/from_leverage", "POST", data=data)

    def transfer_from_leverage_account(self, amount, currency="JPY"):
        """
        transfer money from a leverage account to an account for spot tradings
        :param amount: amount to transfer
        :param currency: currency to transfer. As of 2018-05, only "JPY" is supported
        :return: succeeded or not
        """
        if currency not in ["JPY"]:
            raise CoinCheckApiException("currency should be \"JPY\", currently.")
        data = {"amount": amount, "currency": currency}
        return self.execute_private_api("/api/exchange/transfers/from_leverage", "POST", data=data)
